@@ -12,6 +12,7 @@ from lintian_brush import (
     min_certainty,
     )
 from lintian_brush.upstream_metadata import (
+    UpstreamDatum,
     check_upstream_metadata,
     extend_upstream_metadata,
     guess_upstream_metadata_items,
@@ -37,25 +38,24 @@ except FileNotFoundError:
 else:
     code = ruamel.yaml.round_trip_load(inp, preserve_quotes=True)
 
-original_fields = set(code)
+upstream_metadata = {
+    k: UpstreamDatum(k, v, 'certain') for (k, v) in code.items()}
 
 minimum_certainty = os.environ.get('MINIMUM_CERTAINTY')
-fields = set()
-current_certainty = {k: 'certain' for k in code.keys()}
 
-fields.update(update_from_guesses(
-    code, current_certainty, guess_upstream_metadata_items(
-        '.', trust_package=(os.environ.get('TRUST_PACKAGE') == 'true'))))
+update_from_guesses(
+    upstream_metadata,
+    guess_upstream_metadata_items(
+        '.', trust_package=(os.environ.get('TRUST_PACKAGE') == 'true')))
 
 net_access = os.environ.get('NET_ACCESS', 'allow') == 'allow'
-fields.update(extend_upstream_metadata(
-    code, current_certainty, '.',
+extend_upstream_metadata(
+    upstream_metadata, '.',
     minimum_certainty=minimum_certainty, net_access=net_access,
-    consult_external_directory=True))
+    consult_external_directory=True)
 if net_access:
-    # TODO(jelmer): Set package
     check_upstream_metadata(
-        code, current_certainty, version=current_version.upstream_version)
+        upstream_metadata, version=current_version.upstream_version)
 
 # Homepage is set in debian/control, so don't add it to
 # debian/upstream/metadata.
@@ -64,39 +64,34 @@ external_present_fields = set(['Homepage'])
 # If the debian/copyright file is machine-readable, then we do
 # not need to set the Name/Contact information in the debian/upstream/metadata
 # file.
-if 'Name' in code or 'Contact' in code:
+if 'Name' in upstream_metadata or 'Contact' in upstream_metadata:
     from lintian_brush.copyright import upstream_fields_in_copyright
     external_present_fields.update(upstream_fields_in_copyright())
 
 
-# Drop keys that don't need to be in debian/upsteam/metadata
-for key in list(code):
-    if key in original_fields:
-        continue
+for key, datum in list(upstream_metadata.items()):
+    # Drop keys that don't need to be in debian/upsteam/metadata
     if key.startswith('X-') or key in external_present_fields:
-        del code[key]
-        del current_certainty[key]
-        if key in fields:
-            fields.remove(key)
+        del upstream_metadata[key]
+
+    # Drop everything that is below our minimum certainty
+    if not certainty_sufficient(datum.certainty, minimum_certainty):
+        del upstream_metadata[key]
 
 
-# Drop everything that is below our minimum certainty
-for key, certainty in list(current_certainty.items()):
-    if not certainty_sufficient(certainty, minimum_certainty):
-        del code[key]
-        del current_certainty[key]
-        if key in fields:
-            fields.remove(key)
+achieved_certainty = min_certainty(
+    [d.certainty for d in upstream_metadata.values()])
 
-achieved_certainty = min_certainty(current_certainty.values())
+if 'Repository' in upstream_metadata:
+    repo = upstream_metadata['Repository']
+    repo.value = sanitize_vcs_url(repo.value)
 
-if 'Repository' in code:
-    new_repository = sanitize_vcs_url(code['Repository'])
-    if new_repository != code['Repository']:
-        code['Repository'] = new_repository
-        fields.add('Repository')
+changed = {
+    k: v
+    for k, v in upstream_metadata.items()
+    if v.value != code.get(k)}
 
-if not fields:
+if not changed:
     sys.exit(0)
 
 if not os.path.isdir('debian/upstream'):
@@ -104,8 +99,26 @@ if not os.path.isdir('debian/upstream'):
 
 fixed_tag = not os.path.exists('debian/upstream/metadata')
 
+
+# If we're setting them new, put Name and Contact first
+def sort_key(x):
+    (k, v) = x
+    return {
+        'Name': '00-Name',
+        'Contact': '01-Contact',
+        }.get(k, k)
+
+
+for k, v in sorted(changed.items(), key=sort_key):
+    code[k] = v.value
+
 write_yaml_file('debian/upstream/metadata', code)
 
+fields = [
+    ('%s (from %s)' % (v.field, v.origin)) if v.origin else v.field
+    for k, v in sorted(changed.items())
+    ]
+
 print('Set upstream metadata fields: %s.' % ', '.join(sorted(fields)))
 print('Certainty: %s' % achieved_certainty)
 if fixed_tag:
