@@ -3,7 +3,7 @@
 import os
 import ruamel.yaml
 
-OBSOLETE_FIELDS = ['Name', 'Contact']
+obsolete_fields = set()
 removed_fields = []
 
 try:
@@ -14,7 +14,14 @@ except FileNotFoundError:
 else:
     code = ruamel.yaml.round_trip_load(inp, preserve_quotes=True)
 
-for field in OBSOLETE_FIELDS:
+# If the debian/copyright file is machine-readable, then we can drop the
+# Name/Contact information from the debian/upstream/metadata file.
+if 'Name' in code or 'Contact' in code:
+    from lintian_brush.copyright import upstream_fields_in_copyright
+    obsolete_fields.update(upstream_fields_in_copyright('debian/copyright'))
+
+
+for field in obsolete_fields:
     if field in code:
         del code[field]
         removed_fields.append(field)
@@ -29,4 +36,4 @@ if removed_fields:
             os.rmdir('debian/upstream')
 
 print('Remove obsolete fields %s from debian/upstream/metadata.' %
-      ', '.join(removed_fields))
+      ', '.join(sorted(removed_fields)))
