@@ -31,6 +31,7 @@ from debian.deb822 import Deb822
 from lintian_brush import (
     USER_AGENT,
     DEFAULT_URLLIB_TIMEOUT,
+    SUPPORTED_CERTAINTIES,
     certainty_sufficient,
     certainty_to_confidence,
     )
@@ -48,6 +49,35 @@ KNOWN_HOSTING_SITES = [
     'code.launchpad.net', 'github.com', 'launchpad.net']
 
 
+class UpstreamDatum(object):
+    """A single piece of upstream metadata."""
+
+    __slots__ = ['field', 'value', 'certainty', 'origin']
+
+    def __init__(self, field, value, certainty=None, origin=None):
+        self.field = field
+        self.value = value
+        if certainty not in SUPPORTED_CERTAINTIES:
+            raise ValueError(certainty)
+        self.certainty = certainty
+        self.origin = origin
+
+    def __eq__(self, other):
+        return isinstance(other, type(self)) and \
+                self.field == other.field and \
+                self.value == other.value and \
+                self.certainty == other.certainty and \
+                self.origin == other.origin
+
+    def __str__(self):
+        return "%s: %s" % (self.field, self.value)
+
+    def __repr__(self):
+        return "%s(%r, %r, %r, %r)" % (
+            type(self).__name__, self.field, self.value, self.certainty,
+            self.origin)
+
+
 # See https://wiki.debian.org/UpstreamMetadata
 # Supported fields:
 # - Homepage
@@ -96,10 +126,11 @@ def get_sf_metadata(project):
         raise NoSuchSourceForgeProject(project)
 
 
-def guess_repo_from_url(url):
+def guess_repo_from_url(url, net_access=False):
     parsed_url = urlparse(url)
+    path_elements = parsed_url.path.strip('/').split('/')
     if parsed_url.netloc == 'github.com':
-        if parsed_url.path.strip('/').count('/') < 1:
+        if len(path_elements) < 2:
             return None
         return ('https://github.com' +
                 '/'.join(parsed_url.path.split('/')[:3]))
@@ -107,43 +138,41 @@ def guess_repo_from_url(url):
         return 'https://code.launchpad.net/%s' % (
             parsed_url.path.strip('/').split('/')[0])
     if parsed_url.netloc == 'git.savannah.gnu.org':
-        path_elements = parsed_url.path.strip('/').split('/')
         if len(path_elements) != 2 or path_elements[0] != 'git':
             return None
         return url
     if parsed_url.netloc in ('freedesktop.org', 'www.freedesktop.org'):
-        path_elements = parsed_url.path.strip('/').split('/')
         if len(path_elements) >= 2 and path_elements[0] == 'software':
             return 'https://github.com/freedesktop/%s' % path_elements[1]
         if len(path_elements) >= 3 and path_elements[:2] == [
                 'wiki', 'Software']:
             return 'https://github.com/freedesktop/%s.git' % path_elements[2]
     if parsed_url.netloc == 'download.gnome.org':
-        path_elements = parsed_url.path.strip('/').split('/')
         if len(path_elements) >= 2 and path_elements[0] == 'sources':
             return 'https://gitlab.gnome.org/gnome/%s.git' % path_elements[1]
     if parsed_url.netloc == 'ftp.gnome.org':
-        path_elements = parsed_url.path.strip('/').split('/')
         if (len(path_elements) >= 4 and [
               e.lower() for e in path_elements[:3]] == [
                   'pub', 'gnome', 'sources']):
             return 'https://gitlab.gnome.org/gnome/%s.git' % path_elements[3]
     if parsed_url.netloc == 'sourceforge.net':
-        path_elements = parsed_url.path.strip('/').split('/')
         if (len(path_elements) >= 4 and path_elements[0] == 'p'
                 and path_elements[3] == 'ci'):
             return 'https://sourceforge.net/p/%s/%s' % (
                 path_elements[1], path_elements[2])
     if parsed_url.netloc == 'www.apache.org':
-        path_elements = parsed_url.path.strip('/').split('/')
         if len(path_elements) > 2 and path_elements[0] == 'dist':
             return 'https://svn.apache.org/repos/asf/%s/%s' % (
                 path_elements[1], path_elements[2])
     if parsed_url.netloc == 'bitbucket.org':
-        path_elements = parsed_url.path.strip('/').split('/')
         if len(path_elements) >= 2:
             return 'https://bitbucket.org/%s/%s' % (
                 path_elements[0], path_elements[1])
+    if parsed_url.netloc == 'ftp.gnu.org':
+        if len(path_elements) >= 2 and path_elements[0] == 'gnu':
+            return 'https://git.savannah.gnu.org/git/%s.git' % (
+                path_elements[1])
+        return None
     if parsed_url.netloc in KNOWN_GITLAB_SITES:
         if parsed_url.path.strip('/').count('/') < 1:
             return None
@@ -154,20 +183,19 @@ def guess_repo_from_url(url):
             parsed_url._replace(path='/'.join(parts), query=''))
     if parsed_url.netloc in KNOWN_HOSTING_SITES:
         return url
+    # Maybe it's already pointing at a VCS repo?
+    if net_access and probe_upstream_branch_url(url):
+        return url
     return None
 
 
-def update_from_guesses(code, current_certainty, guessed_items):
-    fields = set()
-    for key, value, certainty in guessed_items:
-        if key not in current_certainty or (
-                certainty_to_confidence(certainty) <
-                certainty_to_confidence(current_certainty[key])):
-            if code.get(key) != value:
-                code[key] = value
-                fields.add(key)
-            current_certainty[key] = certainty
-    return fields
+def update_from_guesses(upstream_metadata, guessed_items):
+    for datum in guessed_items:
+        current_datum = upstream_metadata.get(datum.field)
+        if not current_datum or (
+                certainty_to_confidence(datum.certainty) <
+                certainty_to_confidence(current_datum.certainty)):
+            upstream_metadata[datum.field] = datum
 
 
 def read_python_pkg_info(path):
@@ -222,24 +250,27 @@ def guess_from_debian_watch(path, trust_package):
             if url.startswith('https://') or url.startswith('http://'):
                 repo = guess_repo_from_url(url)
                 if repo:
-                    yield "Repository", sanitize_vcs_url(repo), "likely"
+                    yield UpstreamDatum(
+                        "Repository", sanitize_vcs_url(repo), "likely")
                     break
             m = re.match('https?://sf.net/([^/]+)', url)
             if m:
-                yield "Archive", "SourceForge", "certain"
-                yield "X-SourceForge-Project", m.group(1), "certain"
+                yield UpstreamDatum("Archive", "SourceForge", "certain")
+                yield UpstreamDatum(
+                    "X-SourceForge-Project", m.group(1), "certain")
 
 
 def guess_from_debian_control(path, trust_package):
     with open(path, 'r') as f:
         control = Deb822(f)
     if 'Homepage' in control:
-        yield 'Homepage', control['Homepage'], 'certain'
+        yield UpstreamDatum('Homepage', control['Homepage'], 'certain')
     if 'XS-Go-Import-Path' in control:
         yield (
-            'Repository',
-            sanitize_vcs_url('https://' + control['XS-Go-Import-Path']),
-            'likely')
+            UpstreamDatum(
+                'Repository',
+                sanitize_vcs_url('https://' + control['XS-Go-Import-Path']),
+                'likely'))
 
 
 def guess_from_setup_py(path, trust_package):
@@ -250,15 +281,17 @@ def guess_from_setup_py(path, trust_package):
         pass
     else:
         if pkg_info.name:
-            yield 'Name', pkg_info.name, 'certain'
+            yield UpstreamDatum('Name', pkg_info.name, 'certain')
         if pkg_info.home_page:
             repo = guess_repo_from_url(pkg_info.home_page)
             if repo:
-                yield 'Repository', sanitize_vcs_url(repo), 'likely'
+                yield UpstreamDatum(
+                    'Repository', sanitize_vcs_url(repo), 'likely')
         for value in pkg_info.project_urls:
             url_type, url = value.split(', ')
             if url_type in ('GitHub', 'Repository', 'Source Code'):
-                yield 'Repository', sanitize_vcs_url(url), 'certain'
+                yield UpstreamDatum(
+                    'Repository', sanitize_vcs_url(url), 'certain')
 
 
 def guess_from_package_json(path, trust_package):
@@ -266,9 +299,9 @@ def guess_from_package_json(path, trust_package):
     with open(path, 'r') as f:
         package = json.load(f)
     if 'name' in package:
-        yield 'Name', package['name'], 'certain'
+        yield UpstreamDatum('Name', package['name'], 'certain')
     if 'homepage' in package:
-        yield 'Homepage', package['homepage'], 'certain'
+        yield UpstreamDatum('Homepage', package['homepage'], 'certain')
     if 'repository' in package:
         if isinstance(package['repository'], dict):
             repo_url = package['repository'].get('url')
@@ -279,18 +312,20 @@ def guess_from_package_json(path, trust_package):
         if repo_url:
             parsed_url = urlparse(repo_url)
             if parsed_url.scheme and parsed_url.netloc:
-                yield 'Repository', sanitize_vcs_url(repo_url), 'certain'
+                yield UpstreamDatum(
+                    'Repository', sanitize_vcs_url(repo_url), 'certain')
             else:
                 # Some people seem to default to github. :(
                 repo_url = 'https://github.com/' + parsed_url.path
-                yield 'Repository', sanitize_vcs_url(repo_url), 'likely'
+                yield UpstreamDatum(
+                    'Repository', sanitize_vcs_url(repo_url), 'likely')
     if 'bugs' in package:
         if isinstance(package['bugs'], dict):
             url = package['bugs'].get('url')
         else:
             url = package['bugs']
         if url:
-            yield 'Bugs-Database', url, 'certain'
+            yield UpstreamDatum('Bugs-Database', url, 'certain')
 
 
 def guess_from_package_xml(path, trust_package):
@@ -307,12 +342,13 @@ def guess_from_package_xml(path, trust_package):
         ), 'root tag is %r' % root.tag
     name_tag = root.find('name')
     if name_tag is not None:
-        yield 'Name', name_tag.text, 'certain'
+        yield UpstreamDatum('Name', name_tag.text, 'certain')
     for url_tag in root.findall('url'):
         if url_tag.get('type') == 'repository':
-            yield 'Repository', sanitize_vcs_url(url_tag.text), 'certain'
+            yield UpstreamDatum(
+                'Repository', sanitize_vcs_url(url_tag.text), 'certain')
         if url_tag.get('type') == 'bugtracker':
-            yield 'Bug-Database', url_tag.text, 'certain'
+            yield UpstreamDatum('Bug-Database', url_tag.text, 'certain')
 
 
 def guess_from_dist_ini(path, trust_package):
@@ -329,17 +365,18 @@ def guess_from_dist_ini(path, trust_package):
         except ParsingError as e:
             warn('Unable to parse dist.ini: %r' % e)
     try:
-        yield 'Name', parser['START']['name'], 'certain'
+        yield UpstreamDatum('Name', parser['START']['name'], 'certain')
     except (NoSectionError, NoOptionError, KeyError):
         pass
     try:
-        yield ('Bug-Database',
-               parser['MetaResources']['bugtracker.web'], 'certain')
+        yield UpstreamDatum(
+            'Bug-Database', parser['MetaResources']['bugtracker.web'],
+            'certain')
     except (NoSectionError, NoOptionError, KeyError):
         pass
     try:
-        yield ('Repository',
-               parser['MetaResources']['repository.url'], 'certain')
+        yield UpstreamDatum(
+            'Repository', parser['MetaResources']['repository.url'], 'certain')
     except (NoSectionError, NoOptionError, KeyError):
         pass
 
@@ -367,9 +404,10 @@ def guess_from_debian_copyright(path, trust_package):
             header = copyright.header
     if header:
         if header.upstream_name:
-            yield "Name", header.upstream_name, 'certain'
+            yield UpstreamDatum("Name", header.upstream_name, 'certain')
         if header.upstream_contact:
-            yield "Contact", ','.join(header.upstream_contact), 'certain'
+            yield UpstreamDatum(
+                "Contact", ','.join(header.upstream_contact), 'certain')
         if header.source:
             if ' 'in header.source:
                 from_url = re.split('[ ,]', header.source)[0]
@@ -377,12 +415,16 @@ def guess_from_debian_copyright(path, trust_package):
                 from_url = header.source
             repo_url = guess_repo_from_url(from_url)
             if repo_url:
-                yield 'Repository', sanitize_vcs_url(repo_url), 'likely'
+                yield UpstreamDatum(
+                    'Repository', sanitize_vcs_url(repo_url), 'likely')
         if "X-Upstream-Bugs" in header:
-            yield "Bug-Database", header["X-Upstream-Bugs"], 'certain'
+            yield UpstreamDatum(
+                "Bug-Database", header["X-Upstream-Bugs"], 'certain')
         if "X-Source-Downloaded-From" in header:
-            yield "Repository", guess_repo_from_url(
-                header["X-Source-Downloaded-From"]), 'certain'
+            yield UpstreamDatum(
+                "Repository",
+                guess_repo_from_url(header["X-Source-Downloaded-From"]),
+                'certain')
 
 
 def guess_from_readme(path, trust_package):
@@ -410,8 +452,14 @@ def guess_from_readme(path, trust_package):
                 m = re.match(
                     b'.*\\(https://travis-ci.org/([^/]+)/([^/]+)\\)', line)
                 if m:
-                    yield 'Repository', 'https://github.com/%s/%s' % (
-                        m.group(1).decode(), m.group(2).decode()), 'possible'
+                    yield UpstreamDatum(
+                        'Repository', 'https://github.com/%s/%s' % (
+                            m.group(1).decode(), m.group(2).decode()),
+                        'possible')
+                for m in re.finditer(
+                        b'https://github.com/([^/]+)/([^/]+)/issues', line):
+                    yield UpstreamDatum(
+                        'Bug-Database', m.group(0).decode(), 'possible')
     except IsADirectoryError:
         pass
 
@@ -422,7 +470,7 @@ def guess_from_readme(path, trust_package):
         return 0
     urls.sort(key=prefer_public)
     if urls:
-        yield ('Repository', urls[0], 'possible')
+        yield UpstreamDatum('Repository', urls[0], 'possible')
 
 
 def guess_from_meta_json(path, trust_package):
@@ -430,21 +478,24 @@ def guess_from_meta_json(path, trust_package):
     with open(path, 'r') as f:
         data = json.load(f)
         if 'name' in data:
-            yield 'Name', data['name'], 'certain'
+            yield UpstreamDatum('Name', data['name'], 'certain')
         if 'resources' in data:
             resources = data['resources']
             if 'bugtracker' in resources and 'web' in resources['bugtracker']:
-                yield "Bug-Database", resources["bugtracker"]["web"], 'certain'
+                yield UpstreamDatum(
+                    "Bug-Database", resources["bugtracker"]["web"], 'certain')
                 # TODO(jelmer): Support resources["bugtracker"]["mailto"]
             if 'homepage' in resources:
-                yield "Homepage", resources["homepage"], 'certain'
+                yield UpstreamDatum(
+                    "Homepage", resources["homepage"], 'certain')
             if 'repository' in resources:
                 repo = resources['repository']
                 if 'url' in repo:
-                    yield (
+                    yield UpstreamDatum(
                         'Repository', sanitize_vcs_url(repo["url"]), 'certain')
                 if 'web' in repo:
-                    yield 'Repository-Browse', repo['web'], 'certain'
+                    yield UpstreamDatum(
+                        'Repository-Browse', repo['web'], 'certain')
 
 
 def guess_from_meta_yml(path, trust_package):
@@ -462,20 +513,23 @@ def guess_from_meta_yml(path, trust_package):
             warn('Unable to parse META.yml: %s' % e)
             return
         if 'name' in data:
-            yield 'Name', data['name'], 'certain'
+            yield UpstreamDatum('Name', data['name'], 'certain')
         if 'resources' in data:
             resources = data['resources']
             if 'bugtracker' in resources:
-                yield 'Bug-Database', resources['bugtracker'], 'certain'
+                yield UpstreamDatum(
+                    'Bug-Database', resources['bugtracker'], 'certain')
             if 'homepage' in resources:
-                yield 'Homepage', resources['homepage'], 'certain'
+                yield UpstreamDatum(
+                    'Homepage', resources['homepage'], 'certain')
             if 'repository' in resources:
                 if isinstance(resources['repository'], dict):
                     url = resources['repository'].get('url')
                 else:
                     url = resources['repository']
                 if url:
-                    yield ('Repository', sanitize_vcs_url(url), 'certain')
+                    yield UpstreamDatum(
+                        'Repository', sanitize_vcs_url(url), 'certain')
 
 
 def guess_from_doap(path, trust_package):
@@ -499,15 +553,15 @@ def guess_from_doap(path, trust_package):
 
     for child in root:
         if child.tag == ('{%s}name' % DOAP_NAMESPACE):
-            yield 'Name', child.text, 'certain'
+            yield UpstreamDatum('Name', child.text, 'certain')
         if child.tag == ('{%s}bug-database' % DOAP_NAMESPACE):
             url = extract_url(child)
             if url:
-                yield 'Bug-Database', url, 'certain'
+                yield UpstreamDatum('Bug-Database', url, 'certain')
         if child.tag == ('{%s}homepage' % DOAP_NAMESPACE):
             url = extract_url(child)
             if url:
-                yield 'Homepage', url, 'certain'
+                yield UpstreamDatum('Homepage', url, 'certain')
         if child.tag == ('{%s}repository' % DOAP_NAMESPACE):
             for repo in child:
                 if repo.tag in (
@@ -520,7 +574,7 @@ def guess_from_doap(path, trust_package):
                     else:
                         repo_url = None
                     if repo_url:
-                        yield (
+                        yield UpstreamDatum(
                             'Repository', sanitize_vcs_url(repo_url),
                             'certain')
                     web_location = repo.find(
@@ -531,7 +585,8 @@ def guess_from_doap(path, trust_package):
                         web_url = None
 
                     if web_url:
-                        yield 'Repository-Browse', web_url, 'certain'
+                        yield UpstreamDatum(
+                            'Repository-Browse', web_url, 'certain')
 
 
 def guess_from_configure(path, trust_package=False):
@@ -550,22 +605,27 @@ def guess_from_configure(path, trust_package=False):
             if not value:
                 continue
             if key == b'PACKAGE_NAME':
-                yield 'Name', value.decode(), 'certain'
+                yield UpstreamDatum(
+                    'Name', value.decode(), 'certain', './configure')
             elif key == b'PACKAGE_BUGREPORT':
-                yield 'Bug-Submit', value.decode(), 'certain'
+                yield UpstreamDatum(
+                    'Bug-Submit', value.decode(), 'certain', './configure')
             elif key == b'PACKAGE_URL':
-                yield 'Homepage', value.decode(), 'certain'
+                yield UpstreamDatum(
+                    'Homepage', value.decode(), 'certain', './configure')
 
 
 def guess_from_r_description(path, trust_package=False):
     with open(path, 'r') as f:
         description = Deb822(f)
         if 'Package' in description:
-            yield 'Name', description['Package'], 'certain'
+            yield UpstreamDatum('Name', description['Package'], 'certain')
         if 'Repository' in description:
-            yield 'Archive', description['Repository'], 'certain'
+            yield UpstreamDatum(
+                'Archive', description['Repository'], 'certain')
         if 'BugReports' in description:
-            yield 'Bug-Database', description['BugReports'], 'certain'
+            yield UpstreamDatum(
+                'Bug-Database', description['BugReports'], 'certain')
         if 'URL' in description:
             entries = [entry.strip()
                        for entry in re.split('[\n,]', description['URL'])]
@@ -580,21 +640,22 @@ def guess_from_r_description(path, trust_package=False):
                     label = None
                 urls.append((label, url))
             if len(urls) == 1:
-                yield 'Homepage', urls[0][1], 'possible'
+                yield UpstreamDatum('Homepage', urls[0][1], 'possible')
             for label, url in urls:
                 if label and label.lower() in ('devel', 'repository'):
-                    yield 'Repository', url, 'certain'
+                    yield UpstreamDatum('Repository', url, 'certain')
                 elif label and label.lower() in ('homepage', ):
-                    yield 'Homepage', url, 'certain'
+                    yield UpstreamDatum('Homepage', url, 'certain')
                 else:
                     repo_url = guess_repo_from_url(url)
                     if repo_url:
-                        yield 'Repository', repo_url, 'certain'
+                        yield UpstreamDatum('Repository', repo_url, 'certain')
 
 
 def guess_from_environment():
     try:
-        yield 'Repository', os.environ['UPSTREAM_BRANCH_URL'], 'certain'
+        yield UpstreamDatum(
+            'Repository', os.environ['UPSTREAM_BRANCH_URL'], 'certain')
     except KeyError:
         pass
 
@@ -653,10 +714,10 @@ def guess_upstream_metadata_items(path, trust_package=False,
     """
     guessers = _get_guessers(path, trust_package=trust_package)
     for guesser in guessers:
-        for key, value, certainty in guesser:
-            if not certainty_sufficient(certainty, minimum_certainty):
+        for datum in guesser:
+            if not certainty_sufficient(datum.certainty, minimum_certainty):
                 continue
-            yield key, value, certainty
+            yield datum
 
 
 def guess_upstream_metadata(
@@ -672,24 +733,23 @@ def guess_upstream_metadata(
       consult_external_directory: Whether to pull in data
         from external (user-maintained) directories.
     """
-    current_certainty = {}
-    code = {}
+    upstream_metadata = {}
     update_from_guesses(
-        code, current_certainty,
+        upstream_metadata,
         guess_upstream_metadata_items(
             path, trust_package=trust_package))
 
     extend_upstream_metadata(
-        code, current_certainty, path, net_access=net_access,
+        upstream_metadata, path, net_access=net_access,
         consult_external_directory=consult_external_directory)
-    return code
+    return {k: v.value for (k, v) in upstream_metadata.items()}
 
 
-def _possible_fields_missing(code, certainty, fields, field_certainty):
+def _possible_fields_missing(upstream_metadata, fields, field_certainty):
     for field in fields:
-        if field not in code:
+        if field not in upstream_metadata:
             return True
-        if certainty[field] != 'certain':
+        if upstream_metadata[field].certainty != 'certain':
             return True
     else:
         return False
@@ -705,39 +765,46 @@ def guess_from_sf(sf_project):
         screenshot_urls = [s['url'] for s in data['screenshots'] if 'url' in s]
         if screenshot_urls:
             yield ('Screenshots', screenshot_urls)
-    VCS_NAMES = ['bzr', 'hg', 'git', 'cvs']
+    # TODO(jelmer): What about cvs and bzr?
+    VCS_NAMES = ['hg', 'git', 'svn']
     vcs_tools = [
-        tool for tool in data.get('tools', []) if tool['name'] in VCS_NAMES]
+        (tool['name'], tool['url'])
+        for tool in data.get('tools', []) if tool['name'] in VCS_NAMES]
     if len(vcs_tools) == 1:
-        vcs_tool = vcs_tools[0]
-        if vcs_tool['name'] != 'cvs':
-            yield 'Repository', urljoin('https://sf.net/', vcs_tools[0]['url'])
+        (kind, url) = vcs_tools[0]
+        if kind == 'git':
+            url = urljoin('https://git.code.sf.net/', url)
+        elif kind == 'svn':
+            url = urljoin('https://svn.code.sf.net/', url)
+        elif kind == 'hg':
+            url = urljoin('https://hg.code.sf.net/', url)
+        else:
+            raise KeyError(kind)
+        yield 'Repository', url
 
 
 def extend_from_external_guesser(
-        code, certainty, guesser_certainty, guesser_fields, guesser):
-    fields = set()
+        upstream_metadata, guesser_certainty, guesser_fields, guesser):
     if not _possible_fields_missing(
-            code, certainty, guesser_fields, guesser_certainty):
-        return fields
+            upstream_metadata, guesser_fields, guesser_certainty):
+        return
 
-    fields.update(update_from_guesses(
-        code, certainty,
-        [(key, value, guesser_certainty) for (key, value) in guesser]))
-
-    return fields
+    update_from_guesses(
+        upstream_metadata,
+        [UpstreamDatum(key, value, guesser_certainty)
+         for (key, value) in guesser])
 
 
-def extend_from_sf(code, certainty, sf_project):
+def extend_from_sf(upstream_metadata, sf_project):
     # The set of fields that sf can possibly provide:
     sf_fields = ['Homepage', 'Screenshots', 'Name']
 
     return extend_from_external_guesser(
-        code, certainty, certainty['Archive'], sf_fields,
+        upstream_metadata, upstream_metadata['Archive'].certainty, sf_fields,
         guess_from_sf(sf_project))
 
 
-def extend_from_lp(code, certainty, minimum_certainty, package,
+def extend_from_lp(upstream_metadata, minimum_certainty, package,
                    distribution=None, suite=None):
     # The set of fields that Launchpad can possibly provide:
     lp_fields = ['Homepage', 'Repository', 'Name']
@@ -746,10 +813,10 @@ def extend_from_lp(code, certainty, minimum_certainty, package,
     if minimum_certainty and minimum_certainty != 'possible':
         # Don't bother talking to launchpad if we're not
         # speculating.
-        return set()
+        return
 
-    return extend_from_external_guesser(
-        code, certainty, lp_certainty, lp_fields, guess_from_launchpad(
+    extend_from_external_guesser(
+        upstream_metadata, lp_certainty, lp_fields, guess_from_launchpad(
              package, distribution=distribution, suite=suite))
 
 
@@ -804,31 +871,28 @@ def verify_bug_database_url(url):
     return None
 
 
-def extend_upstream_metadata(code, certainty, path, minimum_certainty=None,
+def extend_upstream_metadata(upstream_metadata, path, minimum_certainty=None,
                              net_access=False,
                              consult_external_directory=False):
     """Extend a set of upstream metadata.
     """
-    if 'Homepage' in code:
-        project = extract_sf_project_name(code['Homepage'])
+    if 'Homepage' in upstream_metadata:
+        project = extract_sf_project_name(upstream_metadata['Homepage'].value)
         if project:
-            code['Archive'] = 'SourceForge'
-            certainty['Archive'] = 'likely'
-            code['X-SourceForge-Project'] = project
-            certainty['X-SourceForge-Project'] = 'likely'
+            upstream_metadata['Archive'] = UpstreamDatum(
+                'Archive', 'SourceForge', 'likely')
+            upstream_metadata['X-SourceForge-Project'] = UpstreamDatum(
+                'X-SourceForge-Project', 'project', 'likely')
 
-    fields = set()
-    if (code.get('Archive') == 'SourceForge' and
-            'X-SourceForge-Project' in code and
+    archive = upstream_metadata.get('Archive')
+    if (archive and archive.value == 'SourceForge' and
+            'X-SourceForge-Project' in upstream_metadata and
             net_access):
-        sf_project = code['X-SourceForge-Project']
+        sf_project = upstream_metadata['X-SourceForge-Project'].value
         try:
-            fields.update(extend_from_sf(code, certainty, sf_project))
+            extend_from_sf(upstream_metadata, sf_project)
         except NoSuchSourceForgeProject:
-            del code['X-SourceForge-Project']
-            del certainty['X-SourceForge-Project']
-            if 'X-SourceForge-Project' in fields:
-                fields.remove('X-SourceForge-Project')
+            del upstream_metadata['X-SourceForge-Project']
     if net_access and consult_external_directory:
         try:
             with open(os.path.join(path, 'debian/control'), 'r') as f:
@@ -837,57 +901,74 @@ def extend_upstream_metadata(code, certainty, path, minimum_certainty=None,
             # Huh, okay.
             pass
         else:
-            fields.update(extend_from_lp(
-                code, certainty, minimum_certainty, package))
-    if 'Homepage' in code and 'Repository' not in code:
-        repo = guess_repo_from_url(code['Homepage'])
+            extend_from_lp(upstream_metadata, minimum_certainty, package)
+    if ('Homepage' in upstream_metadata and
+            'Repository' not in upstream_metadata):
+        repo = guess_repo_from_url(
+                upstream_metadata['Homepage'].value, net_access=net_access)
         if repo:
-            code['Repository'] = repo
-            certainty['Repository'] = 'likely'
-            fields.add('Repository')
-    if 'Repository' in code and 'Repository-Browse' not in code:
-        browse_url = browse_url_from_repo_url(code['Repository'])
+            upstream_metadata['Repository'] = UpstreamDatum(
+                'Repository', repo, 'likely')
+    if ('Bug-Database' in upstream_metadata and
+            'Repository' not in upstream_metadata):
+        repo = guess_repo_from_url(
+            upstream_metadata['Bug-Database'].value, net_access=net_access)
+        if repo:
+            upstream_metadata['Repository'] = UpstreamDatum(
+                'Repository', repo, 'likely')
+    if ('Repository' in upstream_metadata and
+            'Repository-Browse' not in upstream_metadata):
+        browse_url = browse_url_from_repo_url(
+                upstream_metadata['Repository'].value)
         if browse_url:
-            code['Repository-Browse'] = browse_url
-            certainty['Repository-Browse'] = certainty['Repository']
-            fields.add('Repository-Browse')
-    if 'Repository-Browse' in code and 'Repository' not in code:
-        repo = guess_repo_from_url(code['Repository-Browse'])
+            upstream_metadata['Repository-Browse'] = UpstreamDatum(
+                'Repository-Browse', browse_url,
+                upstream_metadata['Repository'].certainty)
+    if ('Repository-Browse' in upstream_metadata and
+            'Repository' not in upstream_metadata):
+        repo = guess_repo_from_url(
+            upstream_metadata['Repository-Browse'].value,
+            net_access=net_access)
         if repo:
-            code['Repository'] = repo
-            certainty['Repository'] = certainty['Repository-Browse']
-            fields.add('Repository')
-    if 'Repository' in code and 'Bug-Database' not in code:
-        bug_db_url = guess_bug_database_url_from_repo_url(code['Repository'])
+            upstream_metadata['Repository'] = UpstreamDatum(
+                'Repository', repo,
+                upstream_metadata['Repository-Browse'].certainty)
+    if ('Repository' in upstream_metadata and
+            'Bug-Database' not in upstream_metadata):
+        bug_db_url = guess_bug_database_url_from_repo_url(
+            upstream_metadata['Repository'].value)
         bug_db_certainty = 'likely'
         if bug_db_url and certainty_sufficient(
                 bug_db_certainty, minimum_certainty):
-            code['Bug-Database'] = bug_db_url
-            certainty['Bug-Database'] = bug_db_certainty
-            fields.add('Bug-Database')
-    if 'Bug-Database' in code and 'Bug-Submit' not in code:
+            upstream_metadata['Bug-Database'] = UpstreamDatum(
+                'Bug-Database', bug_db_url, bug_db_certainty)
+    if ('Bug-Database' in upstream_metadata and
+            'Bug-Submit' not in upstream_metadata):
         bug_submit_url = bug_submit_url_from_bug_database_url(
-            code['Bug-Database'])
+            upstream_metadata['Bug-Database'].value)
         if bug_submit_url:
-            code['Bug-Submit'] = bug_submit_url
-            certainty['Bug-Submit'] = certainty['Bug-Database']
-            fields.add('Bug-Submit')
-    # TODO(jelmer): Set Bug-Submit
-    return fields
+            upstream_metadata['Bug-Submit'] = UpstreamDatum(
+                'Bug-Submit', bug_submit_url,
+                upstream_metadata['Bug-Database'].certainty)
 
 
 def probe_upstream_branch_url(url, version=None):
     parsed = urlparse(url)
-    # TODO(jelmer): Disable authentication prompting.
     if parsed.scheme in ('git+ssh', 'ssh', 'bzr+ssh'):
         # Let's not probe anything possibly non-public.
         return None
+    import breezy.ui
     from breezy.branch import Branch
+    old_ui = breezy.ui.ui_factory
+    breezy.ui.ui_factory = breezy.ui.SilentUIFactory()
     try:
         b = Branch.open(url)
         b.last_revision()
         if version is not None:
             tag_names = b.tags.get_tag_dict().keys()
+            if not tag_names:
+                # Uhm, hmm
+                return True
             if version in tag_names:
                 return True
             if 'v%s' % version in tag_names:
@@ -903,26 +984,31 @@ def probe_upstream_branch_url(url, version=None):
     except Exception:
         # TODO(jelmer): Catch more specific exceptions?
         return False
+    finally:
+        breezy.ui.ui_factory = old_ui
 
 
-def check_upstream_metadata(code, certainty, version=None):
+def check_upstream_metadata(upstream_metadata, version=None):
     """Check upstream metadata.
 
     This will make network connections, etc.
     """
-    if 'Repository' in code and certainty['Repository'] == 'likely':
-        if probe_upstream_branch_url(code['Repository'], version=version):
-            certainty['Repository'] = 'certain'
-            derived_browse_url = browse_url_from_repo_url(code['Repository'])
-            if derived_browse_url == code.get('Repository-Browse'):
-                certainty['Repository-Browse'] = certainty['Repository']
+    repository = upstream_metadata.get('Repository')
+    if repository and repository.certainty == 'likely':
+        if probe_upstream_branch_url(repository.value, version=version):
+            repository.certainty = 'certain'
+            derived_browse_url = browse_url_from_repo_url(repository.value)
+            browse_repo = upstream_metadata.get('Repository-Browse')
+            if browse_repo and derived_browse_url == browse_repo.value:
+                browse_repo.certainty = repository.certainty
         else:
             # TODO(jelmer): Remove altogether, or downgrade to a lesser
             # certainty?
             pass
-    if 'Bug-Database' in code and certainty['Bug-Database'] == 'likely':
-        if verify_bug_database_url(code['Bug-Database']):
-            certainty['Bug-Database'] = 'certain'
+    bug_database = upstream_metadata.get('Bug-Database')
+    if bug_database and bug_database.certainty == 'likely':
+        if verify_bug_database_url(bug_database.value):
+            bug_database.certainty = 'certain'
 
 
 def guess_from_launchpad(package, distribution=None, suite=None):
