@@ -133,6 +133,17 @@ def drop_git_username(parsed, branch):
     return None, None
 
 
+def fix_branch_argument(parsed, branch):
+    if parsed.hostname != 'github.com':
+        return None, None
+    # TODO(jelmer): Handle gitlab sites too?
+    path_elements = parsed.path.strip('/').split('/')
+    if len(path_elements) > 2 and path_elements[2] == 'tree':
+        return (parsed._replace(path='/'.join(path_elements[:2])),
+                '/'.join(path_elements[3:]))
+    return None, None
+
+
 def fixup_broken_git_url(url):
     """Attempt to fix up broken Git URLs.
 
@@ -144,7 +155,7 @@ def fixup_broken_git_url(url):
     changed = False
     for fn in [fix_path_in_port, fix_salsa_scheme, fix_salsa_cgit_url,
                fix_salsa_tree_in_url, fix_double_slash, fix_extra_colon,
-               drop_git_username]:
+               drop_git_username, fix_branch_argument]:
         newparsed, newbranch = fn(parsed, branch)
         if newparsed:
             changed = True
